#ifndef lint
static char *rcsid = "$Header: /a/pluto/vol/fileservers/Pluto/Pluto2/src/X500/xd/work/src/query_engine/RCS/error.c,v 1.0 92/11/27 16:57:45 emsrdsm Exp Locker: emsrdsm $";
#endif

/* $Header: /a/pluto/vol/fileservers/Pluto/Pluto2/src/X500/xd/work/src/query_engine/RCS/error.c,v 1.0 92/11/27 16:57:45 emsrdsm Exp Locker: emsrdsm $ */

/*****************************************************************************

  error.c - Misc. functions for handling error lists.

*****************************************************************************/

#include "types.h"
#include "error.h"
#include "util.h"
#include "request.h"

#include "quipu/util.h"
#include "quipu/ds_error.h"

extern LLog *log_stat;

QE_error error_msgs[] =
{
  {QERR_ok, "OK"},
  {QERR_succeeded, "Successful"},
  {QERR_request_failed, "Request failed!"},
  {QERR_referral_error, "Could not contact remote part of directory"},
  {QERR_bad_name, "Bad name supplied"},
  {QERR_internal_limit_reached, "Internal limit reached. Cannot perform"},
  {QERR_bad_value_syntax, "Bad value supplied"},
  {QERR_time_limit_reached, "Time limit reached, can't complete operation"},
  {QERR_size_limit_reached, "Limit encountered, can't access all information"},
  {QERR_admin_limit_reached, "Limit encountered, can't complete operation"},
  {QERR_local_error, "Local directory system error"},
  {QERR_remote_error, "Error in non-local part of the directory"},
  {QERR_no_such_attr, "No such attribute"},
  {QERR_bad_attr_syntax, "Bad attribute value supplied"},
  {QERR_no_such_object, "No such directory entry"},
  {QERR_alias_problem, "Problem with aliased entry "},
  {QERR_security_error, "Security problem"},
  {QERR_service_busy, "Service busy, cannot access"},
  {QERR_service_unavailable, "Service error, cannot perform"},
  {QERR_chaining_required, "Chaining required"},
  {QERR_unable_to_proceed, "Unable to proceed"},
  {QERR_loop_detect, "Directory loop detected"},
  {QERR_ext_unavailable, "Directory service extension unavailable"},
  {QERR_out_of_scope, "Out of scope"},
  {QERR_dit_error, "Error in directory information structure"},
  {QERR_naming_violation, "Entry naming violation"},
  {QERR_oc_violation, "Object class violation"},
  {QERR_not_on_nonleaf, "Not on non leaf entry"},
  {QERR_not_on_rdn, "Not on RDN"},
  {QERR_already_exists, "Attribute or entry already exists"},
  {QERR_affects_mult_dsas, "Affects multiple systems in directory"},
  {QERR_no_oc_mods, "No object class modifications allowed"},
  {QERR_no_name_changes, "No name change to make"}, 
  {QERR_no_such_op, "No such operation"},
  {QERR_too_late, "Operation requested too late"},
  {QERR_cannot_abandon, "Operation couldn't be abandoned"},
  {QERR_nothing_found, "Nothing found"},
  {QERR_internal, "Internal error, cannot perform"},
  {QERR_no_mods_supplied, "No modifications to make"},
  {QERR_cannot_resolve_name, "Can't resolve given entry name"},
  {QERR_invalid_search_class, "Invalid search data supplied"},
  {QERR_null, ""}
};

/*****************************************************************************
 * get_message_of_code()
 *
 * 
 *
 ****************************************************************************/

char *get_message_of_code(code)
     QE_error_code code;
{
  QCardinal count;

  for (count = 0;
       error_msgs[count].error != QERR_null;
       count++)
    if (code == error_msgs[count].error)
      return error_msgs[count].error_message;

  return NULLCP;
} /* get_message_of_code */

/*****************************************************************************
 * char *get_message_from_ds_error()
 *
 * 
 *
 ****************************************************************************/

char *ds_error_message(error)
     struct DSError *error;
{
  PS ps;
  char buffer[LINESIZE];
  char *str, *message;
  
  extern int ds_error();

  if (error == (struct DSError *) NULL)
    return NULLCP;

  if ((ps = ps_alloc(str_open)) == NULLPS)
    return NULLCP;


  if (str_setup(ps, buffer, LINESIZE, 1) == NOTOK)
    return NULLCP;

  ds_error(ps, error);
  *ps->ps_ptr = 0;
  ps_free(ps);

  str = buffer;

  if (*str != '\0')
    message = copy_string(str);
  else
    message = NULLCP;

  return message;
} /* get_message_from_ds_error */
     
/*****************************************************************************
 * add_error_to_request_rec()
 *
 * 
 *
 ****************************************************************************/

void add_error_to_request_rec(request,
			      baseobject,
			      error_type,
			      error,
			      ds_message)
     requestRec request;
     char *baseobject;
     QE_error_code error_type;
     struct DSError *error;
     char *ds_message;
{
  add_error_to_list(&request->errors,
		    baseobject,
		    error_type,
		    error,
		    ds_message);
} /* add_error_to_request_rec */

/*****************************************************************************
 * add_error_to_list()
 *
 * 
 *
 ****************************************************************************/

void add_error_to_list(list_ptr,
		       baseobject,
		       error_type,
		       error,
		       ds_message)
     errorList *list_ptr;
     char *baseobject;
     QE_error_code error_type;
     struct DSError *error;
     char *ds_message;
{
  errorList new_err = error_alloc();

  new_err->baseobject = (baseobject == NULLCP? NULLCP:copy_string(baseobject));
  new_err->error_type = error_type;

  new_err->next = *list_ptr;

  if (error != NULL)
    new_err->ds_message = ds_error_message(error);
  else if (ds_message != NULLCP)
    new_err->ds_message = copy_string(ds_message);
  else
    new_err->ds_message = NULLCP;

  *list_ptr = new_err;
} /* add_error_to_list */

/*****************************************************************************
 * get_log_error_type()
 *
 * 
 *
 ****************************************************************************/

/* ARGSUSED */
QE_error_code get_log_error_type(error, task_id)
     struct DSError *error;
     int task_id;
{
  extern int log_ds_error();

  log_ds_error(error);

  switch (error->dse_type)
    {
    case DSE_LOCALERROR:
      return QERR_local_error;
      
    case DSE_REMOTEERROR:
      return QERR_remote_error;
      
    case DSE_NOERROR:
      return QERR_ok;
      
    case DSE_ATTRIBUTEERROR:
      return QERR_bad_attr_syntax;
      
    case DSE_NAMEERROR:
      return QERR_bad_name;
      
    case DSE_SERVICEERROR:
      return QERR_service_unavailable;
      
    case DSE_REFERRAL:
    case DSE_DSAREFERRAL:
      return QERR_referral_error;
      
    case DSE_ABANDONED:
      return QERR_ok;
      
    case DSE_ABANDON_FAILED:
      return QERR_ok;
      
    case DSE_SECURITYERROR:
      return QERR_security_error;

    case DSE_UPDATEERROR:
      return QERR_request_failed;
      
      default:
      return QERR_ok;
    }
}

/*****************************************************************************
 * error_list_free()
 *
 * 
 *
 ****************************************************************************/

void error_list_free(error_list_ptr)
     errorList *error_list_ptr;
{
  errorList next_errors, errors = *error_list_ptr;

  while (errors != NULLError)
    {
      if (errors->baseobject != NULLCP) free(errors->baseobject);
      next_errors = errors->next;

      if (errors->ds_message != NULLCP)
	(void) free(errors->ds_message);

      free((char *) errors);
      errors = next_errors;
    }

  *error_list_ptr = NULLError;
} /* error_list_free */

/*****************************************************************************
 * error_list_copy()
 *
 * 
 *
 ****************************************************************************/

errorList error_list_copy(list)
     errorList list;
{
  errorList new_list = NULLError, curr_error = NULLError;

  if (list == NULLError)
    return NULLError;

  for (; list != NULLError; list = list->next)
    {
      if (new_list == NULLError)
	new_list = curr_error = error_alloc();
      else
	curr_error = curr_error->next = error_alloc();

      curr_error->error_type = list->error_type;
      
      curr_error->baseobject = (list->baseobject != NULLCP ?
				copy_string(list->baseobject) :
				NULLCP);
      
      curr_error->ds_message = (list->ds_message != NULLCP ?
				copy_string(list->ds_message) :
				NULLCP);

      curr_error->next = NULLError;
    }

  return new_list;
} /* error_list_copy */

