#ifndef lint
static char *rcsid = "$Header: /a/pluto/vol/fileservers/Pluto/Pluto2/src/X500/xd/work/src/query_engine/RCS/update.c,v 1.0 92/11/27 16:58:34 emsrdsm Exp Locker: emsrdsm $";
#endif

/* $Header: /a/pluto/vol/fileservers/Pluto/Pluto2/src/X500/xd/work/src/query_engine/RCS/update.c,v 1.0 92/11/27 16:58:34 emsrdsm Exp Locker: emsrdsm $ */

/*****************************************************************************

  update.c - Encapsulate modify entry, rename entry operations.

*****************************************************************************/

#include "types.h"
#include "util.h"
#include "modify.h"
#include "rename.h"
#include "ds_tasks.h"
#include "update.h"
#include "request.h"

/***************************************************
 *
 * Static procs...
 *
 */

static errorList start_update();
static request_state process_modify_result(), process_rename_result();

/*****************************************************************************
 * get_update_result()
 *
 * 
 *
 ****************************************************************************/

updateResult get_update_result(request_id)
     QCardinal request_id;
{
  requestRec request_record;
  updateResult result;

  if ((request_record = _get_request_of_id(request_id)) == NULLReqRec)
    return NULLUpdateResult;

  result = request_record->UPDATE_REC->result;
  request_record->UPDATE_REC->result = NULLUpdateResult;

  _request_complete(request_id);

  return result;
} /* get_update_result */

/*****************************************************************************
 * do_update()
 *
 * 
 *
 ****************************************************************************/

errorList do_update(id_ptr, baseobject, new_name, mod_attr_list)
     QCardinal *id_ptr;
     char *baseobject, *new_name;
     modifyAttr mod_attr_list;
{
  updateRec update_request;
  requestRec this_request;

  (void) _request_invoked(UPDATE_ENTRY, id_ptr);
  this_request = _get_request_of_id(*id_ptr);

  update_request = this_request->UPDATE_REC;

  update_request->request_id = *id_ptr;

  update_request->baseobject = copy_string(baseobject);
  update_request->new_name = copy_string(new_name);

  return start_update(*id_ptr, update_request, baseobject, mod_attr_list);
} /* do_update */

/*****************************************************************************
 * start_update()
 *
 * 
 *
 ****************************************************************************/

static errorList start_update(request_id,
			      update_record,
			      baseobject,
			      mod_attr_list)
     QCardinal request_id;
     updateRec update_record;
     char *baseobject;
     modifyAttr mod_attr_list;
{
  errorList errors = NULLError, sub_errors, save_errors = NULLError;
  char err_msg[LINESIZE], entry_name[LINESIZE];

  requestRec request_record = _get_request_of_id(request_id);


  if ((sub_errors = do_modify(&(update_record->sub_request_id),
			      baseobject,
			      mod_attr_list))
      != NULLError)
    {
      if (sub_errors->next == NULLError &&
	  sub_errors->error_type == QERR_no_mods_supplied)
	{
	  request_record->errors = sub_errors;

	  if (update_record->new_name != NULLCP)
	    {
	      if ((save_errors = do_rename(&(update_record->sub_request_id),
					   update_record->baseobject,
					   update_record->new_name))
		  != NULLError)
		{
		  errors = save_errors;

		  friendlify(update_record->baseobject, entry_name);
		  
		  while (errors != NULLError)
		    {
		      if (errors->ds_message != NULLCP)
			{
			  (void) sprintf(err_msg,
				      "While renaming entry `%.24s...'\n%s",
					 entry_name,
					 errors->ds_message);
			  
			  (void) free(errors->ds_message);
			  errors->ds_message = copy_string(err_msg);
			}
		      
		      errors = errors->next;
		    }

		  return save_errors;
		}
	      
	      /***************************************************
	       *
	       * Rename request submitted successfully.
	       *
	       */
	      
	      tag_as_sub_request(update_record->sub_request_id,
				 update_record->request_id);
	      
	      return NULLError;
	    }
	  else
	    return save_errors;
	}
      else
	/***************************************************
	 *
	 * The modify request failed!
	 *
	 */
	{
	  _request_complete(request_id);
	  return sub_errors;
	}
    }

  /***************************************************
   *
   * Modify request has been submitted.
   *
   */
  
  tag_as_sub_request(update_record->sub_request_id, request_id);
  
  return NULLError;
} /* start_update */

/*****************************************************************************
 * process_update_request_result()
 *
 * 
 *
 ****************************************************************************/

request_state process_update_request_result(sub_request_id)
     QCardinal sub_request_id;
{
  QCardinal request_id;
  requestRec request_record, sub_request_record;

  sub_request_record = _get_request_of_id(sub_request_id);
  request_id = sub_request_record->owner_request_id;

  request_record = _get_request_of_id(request_id);

  switch (sub_request_record->type)
    {
    case MODIFY_ENTRY:
      
      return process_modify_result(request_record, sub_request_id);
      
    case RENAME_ENTRY:
      
      return process_rename_result(request_record, sub_request_id);

    default:

      return RQ_processing;
    }
} /* process_update_request_result */

/*****************************************************************************
 * process_rename_result()
 *
 * 
 *
 ****************************************************************************/

static request_state process_rename_result(request_record, sub_request_id)
     requestRec request_record;
     QCardinal sub_request_id;
{
  updateRec update_record = request_record->UPDATE_REC;
  renameResult result;

  errorList errors;
  char err_msg[LINESIZE], entry_name[LINESIZE];

  result = get_rename_result(sub_request_id);
  
  friendlify(update_record->baseobject, entry_name);

  if (result->was_successful == FALSE)
    {
      for (errors = result->errors;
           errors != NULLError && errors->next != NULLError;
           errors = errors->next)
        if (errors->ds_message != NULLCP)
	  {
	    (void) sprintf(err_msg,
                           "While renaming entry `%.24s...'\n%s",
                           entry_name,
                           errors->ds_message);

            (void) free(errors->ds_message);
            errors->ds_message = copy_string(err_msg);
	  }

      if (errors != NULLError)
        {
          if (errors->ds_message != NULLCP)
            {
	      (void) sprintf(err_msg,
			     "While renaming entry `%.24s...'...\n%s",
			     entry_name,
			     errors->ds_message);

              (void) free(errors->ds_message);
              errors->ds_message = copy_string(err_msg);
            }

          errors->next = request_record->errors;
          request_record->errors = result->errors;

          result->errors = NULLError;
        }

      free_rename_result(&result);

      update_record->result = update_result_alloc();

      update_record->result->modify_succeeded = TRUE;
      update_record->result->modify_attempted = TRUE;

      update_record->result->rename_attempted = TRUE;
      update_record->result->rename_succeeded = FALSE;
      
      update_record->result->updated_object = NULLCP;
      update_record->result->was_successful = FALSE;

      update_record->result->errors = request_record->errors;
      request_record->errors = NULLError;

      return RQ_error_returned;
    }

  /***************************************************
   *
   * Rename was successful.
   *
   */

  update_record->result = update_result_alloc();
  
  update_record->result->modify_succeeded = TRUE;
  update_record->result->modify_attempted = TRUE;
  
  update_record->result->rename_attempted = TRUE;
  update_record->result->rename_succeeded = TRUE;

  update_record->result->updated_object = copy_string(result->updated_object);
  update_record->result->was_successful = TRUE;
  
  update_record->result->errors = request_record->errors;
  request_record->errors = NULLError;
  
  free_rename_result(&result);
  
  return RQ_results_returned;
} /* process_rename_result */

/*****************************************************************************
 * process_modify_request_result()
 *
 * 
 *
 ****************************************************************************/

static request_state process_modify_result(request_record, sub_request_id)
     requestRec request_record;
     QCardinal sub_request_id;
{
  modifyResult result;
  updateRec update_record = request_record->UPDATE_REC;

  errorList errors, save_errors;
  char err_msg[LINESIZE], entry_name[LINESIZE];

  result = get_modify_result(sub_request_id);

  friendlify(update_record->baseobject, entry_name);

  if (result->was_successful == FALSE)
    {
      for (errors = result->errors;
           errors != NULLError && errors->next != NULLError;
           errors = errors->next)
        if (errors->ds_message != NULLCP)
          {
            (void) sprintf(err_msg,
                           "While modifying entry `%.32s...'\n%s",
                           entry_name,
			   errors->ds_message);

            (void) free(errors->ds_message);
            errors->ds_message = copy_string(err_msg);
          }

      if (errors != NULLError)
        {
          if (errors->ds_message != NULLCP)
            {
            (void) sprintf(err_msg,
                           "While modifying entry `%.32s...'\n%s",
                           entry_name,
			   errors->ds_message);
	      
              (void) free(errors->ds_message);
              errors->ds_message = copy_string(err_msg);
            }

          errors->next = request_record->errors;
          request_record->errors = result->errors;

          result->errors = NULLError;
        }

      free_modify_result(&result);

      update_record->result = update_result_alloc();

      update_record->result->modify_succeeded = FALSE;
      update_record->result->modify_attempted = TRUE;

      update_record->result->rename_attempted = FALSE;
      update_record->result->rename_succeeded = FALSE;

      update_record->result->updated_object = NULLCP;
      update_record->result->was_successful = FALSE;

      update_record->result->errors = request_record->errors;
      request_record->errors = NULLError;

      return RQ_error_returned;
    }
  else
    /***************************************************
     *
     * Modification was successful.
     *
     */
    {
      free_modify_result(&result);
      
      if (update_record->new_name != NULLCP)
	{
	  if ((errors = do_rename(&(update_record->sub_request_id),
				  update_record->baseobject,
				  update_record->new_name))
	      != NULLError)
	    {
	      save_errors = errors;

	      if (errors->error_type == QERR_no_name_changes)
		/***************************************************
		 *
		 * No name changes required so return success.
		 *
		 */
		{
		  update_record->result = update_result_alloc();
		  
		  update_record->result->modify_succeeded = TRUE;
		  update_record->result->modify_attempted = TRUE;
		  
		  update_record->result->rename_attempted = FALSE;
		  update_record->result->rename_succeeded = FALSE;
		  
		  update_record->result->errors = request_record->errors;
		  request_record->errors = NULLError;
		  
		  update_record->result->updated_object = NULLCP;
		  update_record->result->was_successful = TRUE;

		  return RQ_results_returned;
		}
	      
	      while (errors != NULLError)
		{
		  if (errors->ds_message != NULLCP)
		    {
		      (void) sprintf(err_msg,
				     "While renaming entry `%.32s...'\n%s",
				     entry_name,
				     errors->ds_message);
		      
		      (void) free(errors->ds_message);
		      errors->ds_message = copy_string(err_msg);
		    }
		  
		  if (errors->next == NULLError)
		    {
		      errors->next = request_record->errors;
		      break;
		    }
		}
	      
	      request_record->errors = save_errors;
	      
	      update_record->result = update_result_alloc();
	      
	      update_record->result->modify_succeeded = TRUE;
	      update_record->result->modify_attempted = TRUE;
	      
	      update_record->result->rename_attempted = TRUE;
	      update_record->result->rename_succeeded = FALSE;
	      
	      update_record->result->errors = request_record->errors;
	      request_record->errors = NULLError;
	      
	      update_record->result->updated_object = NULLCP;
	      update_record->result->was_successful = FALSE;

	      return RQ_error_returned;
	    }
	  
	  /***************************************************
	   *
	   * Rename request submitted successfully.
	   *
	   */
	  
	  tag_as_sub_request(update_record->sub_request_id,
			     update_record->request_id);

	  return RQ_processing;
	}
      else
	/***************************************************
	 *
	 * Don't need to do a rename.
	 *
	 */
	{
	  update_record->result = update_result_alloc();

	  update_record->result->rename_attempted = FALSE;
	  update_record->result->rename_succeeded = FALSE;

	  update_record->result->modify_attempted = TRUE;
	  update_record->result->modify_succeeded = TRUE;

	  update_record->result->was_successful = TRUE;

	  update_record->result->errors = request_record->errors;
	  request_record->errors = NULLError;

	  update_record->result->updated_object = NULLCP;

	  return RQ_results_returned;
	}
    }
} /* process_modify_request_result */


/*****************************************************************************
 * free_update_rec()
 *
 * 
 *
 ****************************************************************************/

void free_update_rec(record)
     updateRec record;
{
  if (record == NULLUpdateRec)
    return;

  if (record->new_name != NULLCP)
    (void) free(record->new_name);

  if (record->baseobject != NULLCP)
    (void) free(record->baseobject);
  
  if (record->new_baseobject != NULLCP)
    (void) free(record->new_baseobject);

  if (record->result != NULLUpdateResult)
    free_update_result(&(record->result));

  (void) free((char *) record);
} /* free_update_rec */

/*****************************************************************************
 * free_update_result()
 *
 * 
 *
 ****************************************************************************/

void free_update_result(result_ptr)
     updateResult *result_ptr;
{
  updateResult result = *result_ptr;
  
  if (result == NULLUpdateResult)
    return;

  if (result->errors != NULLError)
    error_list_free(&(result->errors));

  (void) free((char *) result);

  *result_ptr = NULLUpdateResult;
} /* free_update_result */
