#ifndef _Xlu_configure_h_
#define _Xlu_configure_h_

#include "defs.h"
#include "windows.h"
#include "layout.h"
#include "actions.h"
#include "xdutil.h"

#include "attrt_list.h"
#include "sequence.h"

/***************************************************
 *
 * Used in configuration of window templates.
 *
 */

typedef enum 
{
  OT_action,
  OT_width,
  OT_height,
  OT_bitmap,
  OT_helpfile,
  OT_label,
  OT_name,
  OT_expand,
  OT_insensitive
} optionType;

typedef struct obj_res
{
  optionType type;
  union
    {
      String action_name;
      Dimension width;
      Dimension height;
      String bitmap_file;
      String help_file;
      String label;
      String name;
      Boolean expand;
      Boolean insensitive;
    } resource;

  struct obj_res *next;
} object_resource, *objectResource;

#define NULLObjRes (objectResource) NULL
#define resource_alloc() (objectResource) XtMalloc(sizeof(object_resource))

/***************************************************
 *
 * Structures for configuring directory operations.
 *
 */

/***************************************************
 *
 * A group of attributes displayed in a modify pane.
 *
 */

typedef struct _attribute_group
{
  String name;
  String description;

  attrTypeList attributes;

  struct _attribute_group *next;
} attribute_group, *attributeGroup;

#define NULLAttrGroup (attributeGroup) NULL
#define attr_group_alloc() (attributeGroup) XtMalloc(sizeof(attribute_group))

/***************************************************
 *
 * This groups objectClasses into a generic entry type.
 *
 */

typedef struct _entry_class
{
  String name;

  AttributeType rdn_type;
  stringCell groups;

  attrTypeList oc_list;
  
  struct _entry_class *next;
} entry_class, *entryClass;

#define NULLEntryClass (entryClass) NULL
#define entry_class_alloc() (entryClass) smalloc(sizeof(entry_class))

/***************************************************
 *
 * Attribute to objectClass rule.
 *
 */

typedef struct _attr_oc_rule
{
  AttributeType attr;
  AttributeType oc;

  struct _attr_oc_rule *next;
} attr_oc_rule, *attrOcRule;

#define NULLAttrOcRule (attrOcRule) NULL
#define attr_oc_alloc() (attrOcRule) smalloc(sizeof(attr_oc_rule))

/***************************************************
 *
 * Attribute value template.
 *
 */

typedef struct _attr_template
{
  AttributeType type;
  
  stringCell class_names;
  stringCell defaults;

  struct _attr_template *next;
} attr_template, *attrTemplate;

#define NULLAttrTemplate (attrTemplate) NULL
#define attr_tmpl_alloc() (attrTemplate) smalloc(sizeof(attr_template))

/***************************************************
 *
 *  Attributes that are to be displayed.
 *
 */

typedef enum {ShowLongest, ShowAll} showValue;

typedef struct _read_attribute
{
  AttributeType type;
  showValue to_show;
  
  struct _read_attribute *next;
} read_attribute, *readAttribute;

#define NULLReadAttribute (readAttribute) NULL
#define read_attr_alloc() (readAttribute) XtMalloc(sizeof(read_attribute))

/***************************************************
 *
 * Friendly names for attributes.
 *
 */

typedef struct _attribute_name
{
  AttributeType type;
  String name;
  
  struct _attribute_name *next;
} attribute_name, *attributeName;

#define NULLAttributeName (attributeName) NULL
#define attr_name_alloc() (attributeName) XtMalloc(sizeof(attribute_name))

/***************************************************
 *
 * Global functions..
 *
 */

void user_tailor();
void parse_win_config();

Cardinal add_template_entry();
void put_layout();

layoutMember add_prim_to_layout();
layoutMember add_branch_to_layout();

void add_to_startup_list();
void add_action();

objectResource put_resource();
translationList new_trans();

void set_entry_group();
void set_entry_class();

entryClass get_class_of_name();
void set_base_class();

void set_attr_oc_rule();
void set_attr_template();

void add_display_attr();
void add_attribute_name();

void add_ufs_search_class();

void add_dn_db();
String get_dn_db();

#endif /* _Xlu_configure_h_ */

