#ifndef lint
static char *rcsid = "$Header: /a/pluto/vol/fileservers/Pluto/Pluto2/src/X500/xd/work/src/RCS/layout.c,v 1.0 92/11/27 17:00:09 emsrdsm Exp Locker: emsrdsm $";
#endif

/* $Header: /a/pluto/vol/fileservers/Pluto/Pluto2/src/X500/xd/work/src/RCS/layout.c,v 1.0 92/11/27 17:00:09 emsrdsm Exp Locker: emsrdsm $ */

/*****************************************************************************

  layout.c - Prcoedures to produce and layout groups of widgets.

*****************************************************************************/

#include "defs.h"
#include "windows.h"
#include "widgets.h"
#include "layout.h"
#include "xutil.h"
#include "objects.h"
#include "actions.h"
#include "configure.h"
#include "y.tab.h"
#include "help.h"

#include <X11/cursorfont.h>
#include <X11/Xutil.h>
#include <X11/Shell.h>
#include <X11/Xaw/Cardinals.h>
#include <X11/Xaw/Viewport.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/Sme.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/SmeLine.h>
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/Dialog.h>
#include <X11/Xaw/List.h>

#include <X11/bitmaps/gray>

/***************************************************
 *
 * Externals.
 *
 */

extern Display *dpy;

extern Cardinal template_number;
extern windowTemplate win_templates[];
extern Widget toplevel;

extern windowObject win_objects[];

/***************************************************
 *
 * Local functions.
 *
 */

static void makeFromTemplateAux();
static Widget makePrimitive();
static void doLayout(), layoutPrimitive();
static layoutMember findPrimitiveAux();
static void findPrimitiveInstancesAux(), findNamedInstancesAux();

/***************************************************
 *
 * Local data.
 *
 */

Widget widget_list[MAXMEMBERS];
Cardinal widget_count;

/*****************************************************************************
 * makeFromTemplate()
 *
 * Make a group of widgets as defined by the given layout
 * tree. If no parent given make one and return a pointer
 * to it.
 * 
 ****************************************************************************/

Widget makeFromTemplate(template_id,
			object_index,
			parent)
     Cardinal template_id;
     Cardinal object_index;
     Widget parent;
{
  Arg args[MAXARGS];
  Widget form;
  windowTemplate template = win_templates[template_id];
  windowObject this_object = win_objects[object_index];
  WidgetTree top_layout;

  /***************************************************
   *
   * If no parent given, create a new parent
   * (= topLevelShell).
   *
   */

  if (parent == NULLWidget)
    {
      parent = XtCreatePopupShell(template->template_name, 
				  topLevelShellWidgetClass,
				  toplevel,
				  args,
				  0);
    }
    

  /***************************************************
   *
   * Create head of widget map of this object.
   *
   */

  top_layout = WidgetTree_alloc();
  top_layout->left_siblings = NULLWidgetTree;
  top_layout->reference = REF_layout;
  top_layout->descendants = NULLWidgetTree;
  

  /***************************************************
   *
   * Call recursive create procedure.
   *
   */

  makeFromTemplateAux(object_index,
		    template_id,
		    template->layout, 
		    &top_layout->descendants,
		    parent, 
		    &form,
		    NULLWidget,
		    NULLWidget,
		    FALSE,
		    FALSE,
		    horizontal);


  /***************************************************
   *
   * Put parent pointer.
   *
   */

  top_layout->this_widget = form;

  /***************************************************
   *
   * Finalise description of this object,
   * i.e. point to widget map and parent shell.
   *
   */

  this_object->widget_layout = top_layout;
  this_object->parent_shell = parent;

  XtSetMappedWhenManaged(parent, FALSE);
  XtRealizeWidget(parent);

  /***************************************************
   *
   *	Force required layout changes to ensure 
   * 	all available space is taken up.
   *
   */

  doLayout(template->layout, top_layout->descendants, form);

  /***************************************************
   *
   * Do this to force proper laying out by form
   * widget (oh dear!).
   *
   */

  XtUnrealizeWidget(parent);
  XtSetMappedWhenManaged(parent, TRUE);
  XtRealizeWidget(parent);

  /***************************************************
   *
   * Initialize action contexts and
   * other data contexts.
   *
   */

  initContexts(template->layout, top_layout->descendants, object_index);

  /***************************************************
   *
   * Return pointer to parent shell.
   *
   */

  return parent;
} /* makeFromTemplate */

/*****************************************************************************
 * makeFromTemplateAux()
 *
 * Recursive procedure to build widgets and a map of widgets from
 * a given layout.
 *
 ****************************************************************************/

static void makeFromTemplateAux(object_id,
				template_id,
				template,
				widget_layout_ptr,
				parent,
				form,
				from_vert,
				from_horiz,
				expand_this,
				follow_expand,
				parent_orient)
     Cardinal object_id;
     Cardinal template_id;
     layoutMember template;
     WidgetTree *widget_layout_ptr;
     Widget parent;
     Widget *form;
     Widget from_vert;
     Widget from_horiz;
     Boolean expand_this;
     Boolean follow_expand;
     orientation parent_orient;
{
  layoutMember curr_layout = template;
  orientation curr_orientation = curr_layout->orient;
  WidgetTree *curr_sibling, *curr_descendant;
  Widget child_form, curr_prim_widget;
  Arg args[MAXARGS];
  Cardinal count;
  Boolean had_child_expand = FALSE;

  /* Create the parent formWidget. */
  /* If this form is placed relative to another, set separation distance */

  count = 0;
  if (XtClass(parent) == formWidgetClass)
    {
      if (from_vert != (Widget) NULL)
	{
	  XtSetArg(args[count], XtNfromVert, from_vert); count++;

	  if (XtClass(from_vert) == formWidgetClass)
	    {
	      XtSetArg(args[count], XtNvertDistance, 0); count++;
	    }	    
	  else
	    {
	      XtSetArg(args[count], XtNvertDistance, 10); count++;
	    }
	}
      else if (XtClass(XtParent(parent)) == formWidgetClass)
	{
	  XtSetArg(args[count], XtNvertDistance, 0); count++;
	}
      
      if (from_horiz != (Widget) NULL)
	{
	  XtSetArg(args[count], XtNfromHoriz, from_horiz); count++;

	  if (XtClass(from_horiz) == formWidgetClass)
	    {
	      XtSetArg(args[count], XtNhorizDistance, 0); count++;
	    }	    
	  else
	    {
	      XtSetArg(args[count], XtNhorizDistance, 10); count++;
	    }
	}
      
      if (expand_this == TRUE)
	{
	  XtSetArg(args[count], XtNleft, XtChainLeft); count++;
	  XtSetArg(args[count], XtNright, XtChainRight); count++;
	  
	  XtSetArg(args[count], XtNtop, XtChainTop); count++;
	  XtSetArg(args[count], XtNbottom, XtChainBottom); count++;
	}
      else
	{
	  if (parent_orient == horizontal)
	    {
	      XtSetArg(args[count], XtNtop, XtChainTop); count++;
	      XtSetArg(args[count], XtNbottom, XtChainTop); count++;
	      
	      /* If already had an 'expand' */
	      if (follow_expand == FALSE)
		{
		  XtSetArg(args[count], XtNleft, XtChainLeft); count++;
		  XtSetArg(args[count], XtNright, XtChainLeft); count++;
		}
	      else
		{
		  XtSetArg(args[count], XtNleft, XtChainRight); count++;
		  XtSetArg(args[count], XtNright, XtChainRight); count++;
		}
	    }
	  else
	    {
	      XtSetArg(args[count], XtNleft, XtChainLeft); count++;
	      XtSetArg(args[count], XtNright, XtChainLeft); count++;
	      
	      /* If already had an 'expand' */
	      if (follow_expand == FALSE)
		{
		  XtSetArg(args[count], XtNtop, XtChainTop); count++;
		  XtSetArg(args[count], XtNbottom, XtChainTop); count++;
		}
	      else
		{
		  XtSetArg(args[count], XtNtop, XtChainBottom); count++;
		  XtSetArg(args[count], XtNbottom, XtChainBottom); count++;
		}
	    }    
	}
    }

  XtSetArg(args[count], XtNresizable, TRUE); count++;
  XtSetArg(args[count], XtNborderWidth, 0); count++;

  *form = XtCreateManagedWidget("compForm",
				formWidgetClass,
				parent,
				args,
				count);


  from_vert = from_horiz = NULLWidget;
  had_child_expand = FALSE;

  curr_sibling = widget_layout_ptr;
  while (curr_layout)
    {
      switch (curr_layout->ref)
	{
	case REF_layout:

	  *curr_sibling = WidgetTree_alloc();
	  (*curr_sibling)->left_siblings = NULLWidgetTree;
	  (*curr_sibling)->descendants = NULLWidgetTree;
	  
	  curr_descendant = &((*curr_sibling)->descendants);
	  
	  makeFromTemplateAux
	    (object_id,
	     template_id, 
	     win_templates[curr_layout->reference.layout_ref_id]->layout,
	     curr_descendant,
	     *form,
	     &child_form,
	     from_vert,
	     from_horiz,
	     curr_layout->expand,
	     had_child_expand,
	     curr_layout->orient);

	  (*curr_sibling)->this_widget = child_form;
	  (*curr_sibling)->reference = REF_layout;
	  curr_sibling = &((*curr_sibling)->left_siblings);
	  
	  if (curr_orientation == horizontal)
	    from_horiz = child_form;
	  else
	    from_vert = child_form;

	  break;

	case REF_primitive:

	  curr_prim_widget = makePrimitive(object_id,
					   curr_layout,
					   *form,
					   from_vert,
					   from_horiz,
					   had_child_expand);

	  *curr_sibling = WidgetTree_alloc();
	  (*curr_sibling)->left_siblings = NULLWidgetTree;
	  (*curr_sibling)->descendants = NULLWidgetTree;
	  
	  (*curr_sibling)->this_widget = curr_prim_widget;
	  (*curr_sibling)->reference = REF_primitive;
	  
	  curr_sibling = &((*curr_sibling)->left_siblings);
	  
	  if (curr_orientation == horizontal)
	    from_horiz = curr_prim_widget;
	  else
	    from_vert = curr_prim_widget;
	  
	  break;
	}
      
      if (curr_layout->expand == TRUE)
	had_child_expand = TRUE;

      curr_layout->template_id = template_id;
      curr_layout = curr_layout->next_layout;
    }
} /* makeFromTemplateAux */

/*****************************************************************************
 * doLayout()
 *
 * 
 *
 ****************************************************************************/

static void doLayout(template, widget_layout, parent)
     layoutMember template;
     WidgetTree widget_layout;
     Widget parent;
{
  orientation curr_orientation;
  layoutMember curr_layout = template, sub_layout;
  WidgetTree curr_widget_layout;
  Widget this_widget;
  Arg args[MAXARGS];
  Cardinal count;
  Dimension width, height;

  curr_widget_layout = widget_layout;
  curr_orientation = curr_layout->orient;

  while (curr_layout)
    {
      this_widget = curr_widget_layout->this_widget;

      if (curr_layout->ref == REF_layout)
	{
	  XawFormDoLayout(this_widget, FALSE);

	  sub_layout =
	    win_templates[curr_layout->reference.layout_ref_id]->layout;
	  
	  doLayout
	    (sub_layout, curr_widget_layout->descendants, this_widget);
	  
	  if (sub_layout->orient == horizontal)
	    {
	      count = 0;
	      XtSetArg(args[count], XtNwidth, &width); count++;
	      XtGetValues(parent, args, count);
	      
	      count = 0;
	      XtSetArg(args[count], XtNwidth, (Dimension) width-2); count++;
	      XtSetValues(this_widget, args, count);
	    }
	  else
	    {
	      count = 0;
	      XtSetArg(args[count], XtNheight, &height); count++;
	      XtGetValues(parent, args, count);
	      
	      count = 0;
	      XtSetArg(args[count], XtNheight, (Dimension) height-2); count++;
	      XtSetValues(this_widget, args, count);
	    }

	  XawFormDoLayout(this_widget, TRUE);
	}
      else
	layoutPrimitive(curr_layout->reference.primitive_id,
				 parent,
				 this_widget,
				 curr_layout,
				 curr_orientation);
      
      curr_widget_layout = curr_widget_layout->left_siblings;
      curr_layout = curr_layout->next_layout;
    }
} /* doLayout */

/*****************************************************************************
 * layoutPrimitive()
 *
 * 
 *
 ****************************************************************************/

/* ARGSUSED */
static void layoutPrimitive(prim_id, parent, primitive, template, orient)
     Cardinal prim_id;
     Widget parent;
     Widget primitive;
     layoutMember template;
     orientation orient;
{
  Arg args[MAXARGS];
  Widget widget;
  char filename[LINESIZE];
  Pixmap pixmap;
  Dimension width, height;
  
  extern char bitmap_directory[];

  XtSetArg(args[0], XtNresizable, TRUE);
  XtSetValues(primitive, args, 1);

  switch (prim_id)
    {
    case STATUS:
    case TITLE:
    case READ_OUT:
    case LAYOUT:
    case ENTRY_LIST:
    case ERROR_VIEW:
    case MOVETO:

      if (template->width || template->height > 0)
	set_widget_dimensions(primitive, template->width, template->height);

      XtSetArg(args[0], XtNresizable, FALSE);
      XtSetValues(primitive, args, 1);
  
      break;

    case LOOKUP:

      if (template->width || template->height > 0)
	set_widget_dimensions(primitive, template->width, template->height);

      XtSetArg(args[0], XtNresizable, FALSE);
      XtSetValues(primitive, args, 1);

      widget = XtNameToWidget(primitive, "typeButton");
      
      (void) strcpy(filename, bitmap_directory);
      (void) strcat(filename, "menu16");

      pixmap = create_bitmap_from_file(filename, &width, &height, widget);
      
      XtSetArg(args[0], XtNleftBitmap, pixmap);
      XtSetValues(widget, args, 1);

      XtAddCallback(widget,
		    XtNdestroyCallback,
		    XLUfreePixmap,
		    (XtPointer) pixmap);

      break;

    case MODIFY:

      XtSetArg(args[0], XtNresizable, FALSE);
      XtSetValues(primitive, args, 1);
  
      widget = XtNameToWidget(primitive, "groupButton");
      
      (void) strcpy(filename, bitmap_directory);
      (void) strcat(filename, "menu16");

      pixmap = create_bitmap_from_file(filename, &width, &height, widget);
      
      XtSetArg(args[0], XtNleftBitmap, pixmap);
      XtSetValues(widget, args, 1);

      XtAddCallback(widget,
		    XtNdestroyCallback,
		    XLUfreePixmap,
		    (XtPointer) pixmap);

      break;

    case HELP:

      if (template->width || template->height > 0)
	set_widget_dimensions(primitive, template->width, template->height);

      XtSetArg(args[0], XtNresizable, FALSE);
      XtSetValues(primitive, args, 1);

      setHelp(primitive, template);
      break;
      
    default: /* default assumes a button */
      if (template->bitmap_file != NULLString)
	{
	  Pixmap graypixmap;
	  Screen *screen = XtScreen(toplevel);

	  create_background_from_bitmap_file(primitive, template->bitmap_file);
	  
	  graypixmap = XCreatePixmapFromBitmapData(dpy,
						   XtWindow(primitive),
						   gray_bits,
						   gray_width,
						   gray_height,
						   BlackPixelOfScreen(screen),
						   WhitePixelOfScreen(screen),
						   DefaultDepthOfScreen(screen)
						   );
    
	  XtSetArg(args[0], XtNinsensitiveBorder, graypixmap);
	  XtSetArg(args[1], XtNresizable, FALSE);
	  XtSetArg(args[2], XtNshapeStyle, XmuShapeRectangle);
	  XtSetValues(primitive, args, 3);

	  XtAddCallback(primitive,
			XtNdestroyCallback,
			XLUfreePixmap,
			(XtPointer) graypixmap);
	}
    }
} /* layoutPrimitive */

/*****************************************************************************
 * makePrimitive()
 *
 * 
 *
 ****************************************************************************/

static Widget makePrimitive(object_id,
			    template,
			    parent,
			    from_vert,
			    from_horiz,
			    follow_expand)
     Cardinal object_id;
     layoutMember template;
     Widget parent;
     Widget from_vert;
     Widget from_horiz;
     Boolean follow_expand;
{
  Arg args[MAXARGS];
  Cardinal count;
  Widget widget = NULLWidget;
  Cardinal prim_id = template->reference.primitive_id;

  switch (prim_id)
    {
    case BUTTON:
      widget = makeButton(parent, object_id, template);
      break;

    case ENTRY_LIST:
      widget = makeListDisplay(parent, object_id, template);
      break;

    case ERROR_VIEW:
      widget = makeErrorViewer(parent, object_id, template);
      break;

    case HELP:
      widget = makeHelpDisplay(parent, object_id, template);
      break;

    case LOOKUP:
      widget = makeSearchGroup(parent, object_id, template);
      break;

    case MODIFY:
      widget = makeModifyWindow(parent, object_id, template);
      break;

    case MOVETO:
      widget = makeMoveto(parent, object_id, template);
      break;

    case READ_OUT:
      widget = makeReadDisplay(parent, object_id, template);
      break;

    case STATUS:
      widget = makeStatusLabel(parent, object_id, template);
      break;

    case TITLE:
      widget = makeTitleBar(parent, object_id, template);
      break;
    }
  
  count = 0;

  if (from_vert != NULLWidget)
    {
      XtSetArg(args[count], XtNfromVert, from_vert); count++;
      XtSetArg(args[count], XtNvertDistance, 10); count++;
    }
  else
    {
      XtSetArg(args[count], XtNvertDistance, 10); count++;
    }

  if (from_horiz != NULLWidget)
    {
      XtSetArg(args[count], XtNfromHoriz, from_horiz); count++;
      XtSetArg(args[count], XtNhorizDistance, 10); count++;
    }
  else
    {
      XtSetArg(args[count], XtNvertDistance, 10); count++;
    }

  if (template->expand == TRUE)
    {
      XtSetArg(args[count], XtNleft, XtChainLeft); count++;
      XtSetArg(args[count], XtNright, XtChainRight); count++;

      XtSetArg(args[count], XtNtop, XtChainTop); count++;
      XtSetArg(args[count], XtNbottom, XtChainBottom); count++;
    }
  else
    {
      if (template->orient == horizontal)
	{
	  if (prim_id != BUTTON)
	    {
	      XtSetArg(args[count], XtNtop, XtChainTop); count++;
	      XtSetArg(args[count], XtNbottom, XtChainBottom); count++;
	    }
	  else
	    {
	      XtSetArg(args[count], XtNtop, XtChainTop); count++;
	      XtSetArg(args[count], XtNbottom, XtChainTop); count++;
	    }

	  /* If already had an 'expand' */
	  if (follow_expand == FALSE)
	    {
	      XtSetArg(args[count], XtNleft, XtChainLeft); count++;
	      XtSetArg(args[count], XtNright, XtChainLeft); count++;
	    }
	  else
	    {
	      XtSetArg(args[count], XtNleft, XtChainRight); count++;
	      XtSetArg(args[count], XtNright, XtChainRight); count++;
	    }
	}
      else
	{
	  if (prim_id != BUTTON)
	    {
	      XtSetArg(args[count], XtNleft, XtChainLeft); count++;
	      XtSetArg(args[count], XtNright, XtChainRight); count++;
	    }
	  else
	    {
	      XtSetArg(args[count], XtNleft, XtChainLeft); count++;
	      XtSetArg(args[count], XtNright, XtChainLeft); count++;
	    }

	  /* If already had an 'expand' */
	  if (follow_expand == FALSE)
	    {
	      XtSetArg(args[count], XtNtop, XtChainTop); count++;
	      XtSetArg(args[count], XtNbottom, XtChainTop); count++;
	    }
	  else
	    {
	      XtSetArg(args[count], XtNtop, XtChainBottom); count++;
	      XtSetArg(args[count], XtNbottom, XtChainBottom); count++;
	    }
	}    
    }

  XtSetValues(widget, args, count);
  return widget;
} /* makePrimitive */

/*****************************************************************************
 * findPrimitive()
 *
 * 
 *
 ****************************************************************************/

layoutMember findPrimitive(object_id, prim_id, prim_widget)
     Cardinal object_id;
     Cardinal prim_id;
     Widget prim_widget;
{
  WidgetTree curr_widget;
  layoutMember curr_member;
  
  curr_member = win_templates[win_objects[object_id]->template_id]->layout;
  curr_widget = win_objects[object_id]->widget_layout->descendants;
  
  return findPrimitiveAux(curr_member, curr_widget, prim_id, prim_widget);
} /* findPrimitive */

/*****************************************************************************
 * findPrimitiveAux()
 *
 * 
 *
 ****************************************************************************/

static layoutMember findPrimitiveAux(curr_member,
				     curr_widget,
				     prim_id,
				     prim_widget)
     layoutMember curr_member;
     WidgetTree curr_widget;
     Cardinal prim_id;
     Widget prim_widget;
{
  layoutMember return_member;

  while (curr_member)
    {
      switch (curr_member->ref)
	{
	case REF_layout:

	  return_member = findPrimitiveAux
	       (win_templates[curr_member->reference.layout_ref_id]->layout,
		curr_widget->descendants,
		prim_id,
		prim_widget);

	  if (return_member != NULLLayout)
	    return return_member;
      
	  break;
	  
	case REF_primitive:
	  if (curr_widget->this_widget == prim_widget)
	    return curr_member;
	  
	  break;
	}
      
      curr_member = curr_member->next_layout;
      curr_widget = curr_widget->left_siblings;
  }
  
  return NULLLayout;
} /* findPrimitiveAux */

/*****************************************************************************
 * findPrimitiveInstances()
 *
 * 
 *
 ****************************************************************************/

WidgetList findPrimitiveInstances(object_id, prim_id)
     Cardinal object_id;
     Cardinal prim_id;
{
  WidgetTree curr_widget;
  layoutMember curr_member;
  Cardinal num_widgets;
  WidgetList return_widgets;

  curr_member = win_templates[win_objects[object_id]->template_id]->layout;
  curr_widget = win_objects[object_id]->widget_layout->descendants;

  widget_count = 0;
  findPrimitiveInstancesAux(prim_id, curr_member, curr_widget);

  if (widget_count == 0) return ((Widget *) NULL);
  
  return_widgets = (WidgetList) XtMalloc(sizeof(Widget) * (widget_count + 1));
  for (num_widgets = 0; num_widgets < widget_count; num_widgets++)
    return_widgets[num_widgets] = widget_list[num_widgets];

  return_widgets[num_widgets] = NULLWidget;

  return return_widgets;
} /* findPrimitiveInstances */

/*****************************************************************************
 * findPrimitiveInstancesAux()
 *
 * 
 *
 ****************************************************************************/

static void findPrimitiveInstancesAux(prim_id, curr_member, curr_widget)
     Cardinal prim_id;
     layoutMember curr_member;
     WidgetTree curr_widget;
{
  while (curr_member != NULLLayout)
    {
      switch (curr_member->ref)
	{
	case REF_layout:

	  findPrimitiveInstancesAux
	    (prim_id,
	     win_templates[curr_member->reference.layout_ref_id]->layout,
	     curr_widget->descendants);

	  break;
	  
	case REF_primitive:

	  if (curr_member->reference.primitive_id == prim_id)
	    widget_list[widget_count++] = curr_widget->this_widget;
	  
	  break;
	}
      
      curr_member = curr_member->next_layout;
      curr_widget = curr_widget->left_siblings;
    }
} /* findPrimitiveInstancesAux */

/*****************************************************************************
 * findNamedInstances()
 *
 * 
 *
 ****************************************************************************/

WidgetList findNamedInstances(object_id, name)
     Cardinal object_id;
     String name;
{
  WidgetTree curr_widget;
  layoutMember curr_member;
  Cardinal num_widgets;
  WidgetList return_widgets;

  curr_member = win_templates[win_objects[object_id]->template_id]->layout;
  curr_widget = win_objects[object_id]->widget_layout->descendants;

  widget_count = 0;
  findNamedInstancesAux(name, curr_member, curr_widget);

  if (widget_count == 0)
    return ((Widget *) NULL);
  
  return_widgets = (WidgetList) XtMalloc(sizeof(Widget) * (widget_count + 1));

  for (num_widgets = 0; num_widgets < widget_count; num_widgets++)
    return_widgets[num_widgets] = widget_list[num_widgets];

  return_widgets[num_widgets] = NULLWidget;

  return return_widgets;
} /* findNamedInstances */

/*****************************************************************************
 * findNamedInstancesAux()
 *
 * 
 *
 ****************************************************************************/

static void findNamedInstancesAux(name, curr_member, curr_widget)
     String name;
     layoutMember curr_member;
     WidgetTree curr_widget;
{
  static String this_name;

  while (curr_member != NULLLayout)
    {
      switch (curr_member->ref)
	{
	case REF_layout:

	  findNamedInstancesAux
	    (name,
	     win_templates[curr_member->reference.layout_ref_id]->layout,
	     curr_widget->descendants);

	  break;
	  
	case REF_primitive:

	  this_name = curr_member->name;

	  if (this_name != NULLString && !lexequ(this_name, name))
	    widget_list[widget_count++] = curr_widget->this_widget;
	  
	  break;
	}
      
      curr_member = curr_member->next_layout;
      curr_widget = curr_widget->left_siblings;
    }
} /* findNamedInstancesAux */
