#ifndef lint
static char *rcsid = "$Header: /a/pluto/vol/fileservers/Pluto/Pluto2/src/X500/xd/work/src/RCS/objects.c,v 1.0 92/11/27 17:00:29 emsrdsm Exp Locker: emsrdsm $";
#endif

/* $Header: /a/pluto/vol/fileservers/Pluto/Pluto2/src/X500/xd/work/src/RCS/objects.c,v 1.0 92/11/27 17:00:29 emsrdsm Exp Locker: emsrdsm $ */

/*****************************************************************************

  objects.c - Routines to manage and build windows.

*****************************************************************************/

#include "defs.h"
#include "windows.h"

#include <X11/cursorfont.h>
#include <X11/Xutil.h>
#include <X11/Shell.h>

#include <X11/Xaw/Viewport.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/Sme.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/SmeLine.h>
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/Dialog.h>
#include <X11/Xaw/List.h>

#include "layout.h"
#include "objects.h"
#include "requests.h"

#include "showentry.h"
#include "showlist.h"

#include "status.h"
#include "errview.h"
#include "update1.h"
#include "xdutil.h"

#include "query.h"

/***************************************************
 *
 * Globals.
 *
 */

windowObject win_objects[MAXWINDOWS];

Cardinal start_template_ids[MAXSTARTS];
Cardinal startup_number = 0;

windowTemplate win_templates[MAXTEMPLATES];
Cardinal template_number = 0;

/***************************************************
 *
 * Externals.
 *
 */

extern Widget toplevel;
extern assocRec request_list;
extern Cardinal request_count;

void exit();

/***************************************************
 *
 * Local functions.
 *
 */

static void unparent_object_children();

/*****************************************************************************
 * set_current_entry()
 *
 * 
 *
 ****************************************************************************/

void set_current_entry(object_id, new_dn)
     Cardinal object_id;
     String new_dn;
{
  windowObject this_object = win_objects[object_id];
  String new_entry;

  new_entry = (new_dn == NULLString? NULLString: XtNewString(new_dn));

  if (this_object->current_entry != NULLString)
    XtFree(this_object->current_entry);

  this_object->current_entry = new_entry;
} /* set_current_entry */

/*****************************************************************************
 * set_current_errors()
 *
 * 
 *
 ****************************************************************************/

void set_current_errors(object_id, errors)
     Cardinal object_id;
     errorList errors;
{
  windowObject this_object = win_objects[object_id];
  errorList new_errors = error_list_copy(errors);

  if (this_object->current_errors != NULLError)
    error_list_free(&(this_object->current_errors));

  this_object->current_errors = new_errors;
} /* set_current_errors */

/*****************************************************************************
 * set_current_list()
 *
 * 
 *
 ****************************************************************************/

void set_current_list(object_id, newlist)
     Cardinal object_id;
     entryList newlist;
{
  windowObject this_object = win_objects[object_id];
  entryList new_entry_list;

  dn_list_copy(newlist, &new_entry_list);

  if (this_object->current_list != NULLEntryList)
    dn_list_free(&this_object->current_list);

  this_object->current_list = new_entry_list;
} /* set_current_list */

/*****************************************************************************
 * set_current_save_list()
 *
 * 
 *
 ****************************************************************************/

void set_current_save_list(object_id, newlist)
     Cardinal object_id;
     entryList newlist;
{
  windowObject this_object = win_objects[object_id];

  if (this_object->current_save_list != NULLEntryList)
    dn_list_free(&this_object->current_save_list);

  dn_list_copy(newlist, &this_object->current_save_list); 
} /* set_current_save_list */

/*****************************************************************************
 * isWaiting()
 *
 * 
 *
 ****************************************************************************/

Boolean isWaiting(object_id)
     Cardinal object_id;
{
  assocRec curr_request;

  for (curr_request = request_list;
       curr_request != NULLAssocRec;
       curr_request = curr_request->next)
    if (curr_request->object_id == object_id)
      return TRUE;

  return FALSE;
} /* isWaiting */

/*****************************************************************************
 * XLUwait()
 *
 * 
 *
 ****************************************************************************/

void XLUwait(object_id)
     Cardinal object_id;
{
  XEvent event;
  extern XtAppContext app_context;

  while (isWaiting(object_id) == TRUE)
    {
      XtAppNextEvent(app_context, &event);
      XtDispatchEvent(&event);
    }
} /* XLUwait */

/*****************************************************************************
 * make_object()
 *
 * 
 *
 ****************************************************************************/

Widget make_object(template_id, parent_shell, parent_id_ptr, child_id_ptr)
     Cardinal template_id;
     Widget parent_shell;
     Cardinal *parent_id_ptr;
     Cardinal *child_id_ptr;
{
  Cardinal windex;
  windowObject this_object, parent;
  childList siblings;

  for (windex = 0; 
       windex < MAXWINDOWS && win_objects[windex] != NULLWinObject;
       windex++)
    ;

  if (windex == MAXWINDOWS)
    {
      if (parent_id_ptr != (Cardinal *) NULL)
	setStatusMessage(*parent_id_ptr,
		  "Maximum number of windows reached! Cannot create any more");

      *child_id_ptr = MAXWINDOWS;

      return NULLWidget;
    }

  this_object = win_objects[windex] = object_alloc();
  
  (void) makeFromTemplate(template_id, windex, parent_shell);

  this_object->current_entry = NULLString;
  this_object->current_errors = NULLError;
  this_object->current_list = this_object->current_save_list = NULLEntryList;

  this_object->template_id = template_id;

  if (parent_id_ptr != (Cardinal *) NULL)
    {
      this_object->parent_object_id = *parent_id_ptr;
      this_object->is_child = TRUE;
      
      parent = win_objects[*parent_id_ptr];
      siblings = parent->children;
      
      parent->children = child_list_alloc();

      parent->children->child_object_id = windex;
      parent->children->left_siblings = siblings;
    }
  else
    {
      this_object->parent_object_id = 0;
      this_object->is_child = FALSE;
    }

  this_object->children = NULLChildList;
  this_object->is_kept = FALSE;

  *child_id_ptr = windex;
  
  return this_object->parent_shell;
} /* make_object */

/*****************************************************************************
 * create_startup_windows()
 *
 * 
 *
 ****************************************************************************/

void create_startup_windows()
{
  Cardinal count, object_id;
  Widget curr_shell;

  extern char username[];


  if (startup_number == 0)
    xluFatal("No start up windows specified in configure files!");


  for (count = 0; count < startup_number; count++)
    {
      (void) make_object(start_template_ids[count],
			 NULLWidget,
			 (Cardinal *) NULL,
			 &object_id);

      win_objects[count]->current_entry = XtNewString(username);

      displayCurrentList(count);
      updateErrorViewer(count);

      curr_shell = win_objects[count]->parent_shell;
      XtPopup(curr_shell, XtGrabNone);

      displayEntryTemplate(NULLWidget, count, 0, 0, NULLTransList);

      doPrimitiveSensitivity(count);
      XLUwait(count);

      displayCurrentEntry(NULLWidget, count, 0, 0, NULLTransList);
    }
} /* create_startup_windows */

/*****************************************************************************
 * free_child_list()
 *
 * 
 *
 ****************************************************************************/

void free_child_list(child_list_ptr)
     childList child_list_ptr;
{
  if (child_list_ptr == NULLChildList)
    return;

  free_child_list(child_list_ptr->left_siblings);
  XtFree((char *)child_list_ptr);
} /* free_child_list */

/*****************************************************************************
 * unparent_children()
 *
 * 
 *
 ****************************************************************************/

static void unparent_object_children(parent_obj_id)
     Cardinal parent_obj_id;
{
  register childList children = win_objects[parent_obj_id]->children;
  register windowObject currchild;

  while (children)
    {
      currchild = win_objects[children->child_object_id]; 
      currchild->parent_object_id = MAXWINDOWS + 1;
      currchild->is_child = FALSE;
      children = children->left_siblings;
    }
} /* unparent_object_children */

/*****************************************************************************
 * free_widget_tree()
 *
 * 
 *
 ****************************************************************************/

void free_widget_tree(widget_tree_ptr)
     WidgetTree widget_tree_ptr;
{
  if (widget_tree_ptr == NULLWidgetTree)
    return;

  free_widget_tree(widget_tree_ptr->descendants);
  free_widget_tree(widget_tree_ptr->left_siblings);

  XtFree((char *) widget_tree_ptr);
} /* free_widget_tree */

/*****************************************************************************
 * delete_win_object()
 *
 * 
 *
 ****************************************************************************/

void delete_win_object(win_obj_id)
     Cardinal win_obj_id;
{
  windowObject win_obj_ptr = win_objects[win_obj_id];
  childList child_of_parent, delete;

  if (win_obj_ptr == NULLWinObject)
    return;

  unparent_object_children(win_obj_id);
 
  if (win_obj_ptr->is_child)
    {
      child_of_parent = win_objects[win_obj_ptr->parent_object_id]->children;
      
      if (child_of_parent != NULLChildList)
	if (child_of_parent->child_object_id == win_obj_id)
	  {
	    win_objects[win_obj_ptr->parent_object_id]->children = 
	      child_of_parent->left_siblings;
	    XtFree((char *) child_of_parent);
	  }

    while (child_of_parent != NULLChildList && 
	   child_of_parent->left_siblings != NULLChildList)
      {
	if (child_of_parent->left_siblings->child_object_id == win_obj_id)
	  {
	    delete = child_of_parent->left_siblings;
	    child_of_parent->left_siblings = delete->left_siblings;
	    XtFree((char *) delete);
	    
	    break;
	  }
	child_of_parent = child_of_parent->left_siblings;
      }
    }

  free_widget_tree(win_obj_ptr->widget_layout);
  free_child_list(win_obj_ptr->children);
  
  if (win_obj_ptr->current_entry != NULLString)
    XtFree((char *) win_obj_ptr->current_entry);

  if (win_obj_ptr->current_list != NULLEntryList) 
    dn_list_free(&win_obj_ptr->current_list);

  if (win_obj_ptr->current_save_list != NULLEntryList) 
    dn_list_free(&win_obj_ptr->current_save_list);

  if (win_obj_ptr->parent_shell)
    {
      if (XtIsRealized(win_obj_ptr->parent_shell))
	{
	  XtUnmapWidget(win_obj_ptr->parent_shell);
	  
	  if (win_obj_ptr->parent_shell != toplevel)
	    XtDestroyWidget(win_obj_ptr->parent_shell);
	}
    }
  
  win_objects[win_obj_id] = NULLWinObject;
  XtFree((char *) win_obj_ptr);
} /* free_win_object */

/*****************************************************************************
 * find_unkept_child()
 *
 * 
 *
 ****************************************************************************/

Cardinal find_unkept_child(parent_obj_id, child_template_id)
     Cardinal parent_obj_id;
     Cardinal child_template_id;
{
  register childList children = win_objects[parent_obj_id]->children;

  while (children)
    {
      if (win_objects[children->child_object_id]->template_id == 
	  child_template_id && 
	  win_objects[children->child_object_id]->is_kept == FALSE)
	return children->child_object_id;
      
      children = children->left_siblings;
    }

  /* Failed ! */
  return (MAXWINDOWS + 1);
} /* find_unkept_child */

/*****************************************************************************
 * find_unkept_object()
 *
 * 
 *
 ****************************************************************************/

Cardinal find_unkept_object(child_template_id)
     Cardinal child_template_id;
{
  windowObject curr_object;
  Cardinal count;

  for (count = 0, curr_object = win_objects[0];
       count < MAXWINDOWS;
       count++, curr_object = win_objects[count])
    {
      if (curr_object != NULLWinObject &&
	  curr_object->template_id == child_template_id &&
	  curr_object->is_kept == FALSE)
	return count;
    }

  /* Failed ! */
  return (MAXWINDOWS + 1);
} /* find_unkept_object */

/*****************************************************************************
 * get_object_id()
 *
 * 
 *
 ****************************************************************************/

Cardinal get_object_id(w)
     Widget w;
{
  Widget shell;
  Cardinal count;
  
  if (w == NULLWidget)
    return 0;
  
  for (shell = w;
       shell != NULLWidget &&
       XtClass(shell) != topLevelShellWidgetClass &&
       shell != toplevel;
       shell = XtParent(shell))
    ;

  for (count = 0; count < MAXWINDOWS; count++)
    if (win_objects[count] != NULLWinObject &&
	win_objects[count]->parent_shell == shell)
      break;

  return count;
} /* get_object_id */

